@@ -27,15 +27,17 @@
 #include "gconf-cell-renderer.h"
 #include "gconf-editor-application.h"
 #include "gconf-key-editor.h"
+#include "gconf-policykit.h"
 #include "gconf-stock-icons.h"
 #include "gconf-util.h"
 #include "gedit-output-window.h"
 #include "gconf-search-dialog.h"
+
 #include <gconf/gconf.h>
-#include <libintl.h>
+#include <glib/gi18n.h>
+#include <gdk/gdkx.h>
 #include <string.h>
-
-
+#include <stdlib.h>
 
 /* Property IDs */
 enum
@@ -56,7 +58,7 @@ static void		gconf_editor_window_get_property	(GObject          *object,
 
 static GConfClient* 	gconf_editor_window_get_client 		(int type);
 
-static GObjectClass *parent_class;
+G_DEFINE_TYPE(GConfEditorWindow, gconf_editor_window, GTK_TYPE_WINDOW)
 
 void
 gconf_editor_window_popup_error_dialog (GtkWindow   *parent,
@@ -94,21 +96,8 @@ gconf_editor_window_go_to (GConfEditorWindow *window,
 	gint depth;
 	gint i, nchildren;
 	char *key;
-	gboolean dir_exists;
-	GError *error = NULL;
 	
 	g_return_if_fail (location != NULL);
-		
-	dir_exists = gconf_client_dir_exists (window->client, location, &error);
-	
-	if (error != NULL) {
-    		gconf_editor_window_popup_error_dialog (GTK_WINDOW (window),"%s", error);
-        	return;
-	}
-	
-	if (!gconf_client_get (window->client, location, NULL) && !dir_exists)
-		return;
-	
 	
 	child_path = gconf_tree_model_get_tree_path_from_gconf_path (
 				GCONF_TREE_MODEL (window->tree_model), location);
@@ -256,6 +245,7 @@ gconf_add_recent_key (GConfEditorWindow *window, const char *key)
 		/* Keep list on a fixed size, remove the last */
 		if (g_slist_length (list) > RECENT_LIST_MAX_SIZE - 1) {
 			tmp = g_slist_last (list);
+			g_free (tmp->data);
 			list = g_slist_delete_link (list, tmp);
 		}
 
@@ -360,8 +350,9 @@ static void
 help_cb (GtkAction *action, GtkWidget *callback_data)
 {
         GError *error = NULL;
-                                                                                
-        gnome_help_display_desktop (NULL, "gconf-editor", "gconf-editor", NULL, &error);
+
+	gtk_show_uri (gtk_widget_get_screen (callback_data),
+		      "ghelp:gconf-editor", gtk_get_current_event_time (), &error);
                                                                                 
         if (error != NULL) {
                 gconf_editor_window_popup_error_dialog (GTK_WINDOW (GCONF_EDITOR_WINDOW (callback_data)),
@@ -383,9 +374,6 @@ gconf_editor_window_output_window_changed (GtkWidget *widget, gchar *key, gpoint
 	gconf_editor_window_go_to (window, key);
 }
 
-
-
-
 static void
 gconf_editor_window_about_window (GtkAction *action, GtkWidget *callback_data)
 {
@@ -399,16 +387,20 @@ gconf_editor_window_about_window (GtkAction *action, GtkWidget *callback_data)
 		"Shaun McCance <shaunm@gnome.org>",
 		NULL
 	};
-	const gchar *translator_credits = _("translator-credits");
 
 	gtk_show_about_dialog (GTK_WINDOW (callback_data),
+#if GTK_CHECK_VERSION (2, 11, 0)
+                              "program-name", _("Configuration Editor"),
+#else
+                              "name", _("Configuration Editor"),
+#endif
 			      "version", VERSION,
 			      "copyright", "Copyright \xc2\xa9 2001-2004 Anders Carlsson",
 			      "comments", _("An editor for the GConf configuration system."),
 			      "authors", authors,
 			      "documenters", documenters,
-			      "translator_credits", strcmp(translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-			      "logo_icon_name", "gconf-editor",
+			      "translator-credits", _("translator-credits"),
+			      "logo-icon-name", "gconf-editor",
 			      NULL);
 }	
 
@@ -648,24 +640,26 @@ gconf_editor_window_row_expanded (GtkTreeView       *tree_view,
   gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (gconfwindow)));
 }
 
+static void
+gconf_editor_popup_policykit_callback (GtkWindow *window, GError *error)
+{
+  if (error)
+    gconf_editor_window_popup_error_dialog (window, _("Could not set value. Error was:\n%s"), error);
+}
+
 static void
 gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback_data)
 {
   GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
   GtkWindow *window = GTK_WINDOW (callback_data);
 
-  GConfClient *client;
-  GError *error = NULL;
   GtkTreeIter iter;
   GConfValue *value;
   char *path = NULL;
+  gboolean can_use_pk;
+  
+  can_use_pk = (gconfwindow->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL);
   
-  client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
-  if (client == NULL) {
-    g_print ("Could not create GConf client\n");
-    return;
-  }
-	
   gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
                                    NULL, &iter);
   gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
@@ -673,13 +667,32 @@ gconf_editor_popup_window_set_as_default (GtkAction *action, GtkWidget *callback
                       GCONF_LIST_MODEL_VALUE_COLUMN, &value,
                       -1);
 
-  gconf_client_set (client, path, value, NULL);
+  if (gconf_util_can_edit_defaults ()) {
+    GConfClient *client;
 
-  gconf_client_suggest_sync (client, &error);
-  if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
+    if (client != NULL) {
+      GError *error = NULL;
+
+      gconf_client_set (client, path, value, NULL);
+      gconf_client_suggest_sync (client, &error);
+      g_object_unref (client);
+
+      if (!error)
+        return;
+
+      if (!can_use_pk)
+        gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    }
+  }
+
+  if (can_use_pk) {
+    gconf_client_suggest_sync (gconfwindow->client, NULL);
+    gconf_pk_set_default_async (path,
+                                GDK_WINDOW_XWINDOW (GTK_WIDGET (window)->window),
+                                (GFunc) gconf_editor_popup_policykit_callback,
+                                g_object_ref (window), g_object_unref);
   }
-  g_object_unref (client);
 }
 
 static void
@@ -688,19 +701,13 @@ gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callba
   GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
   GtkWindow *window = GTK_WINDOW (callback_data);
 
-  GError *error = NULL;
   GtkTreeIter iter;
   GConfValue *value;
   char *path = NULL;
-  GConfClient *client;
+  gboolean can_use_pk;
   
+  can_use_pk = (gconfwindow->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL);
 
-  client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_MANDATORY);
-  if (client == NULL) {
-    g_print ("Could not create GConf client\n");
-    return;
-  }
-	
   gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
                                    NULL, &iter);
   gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
@@ -708,13 +715,32 @@ gconf_editor_popup_window_set_as_mandatory (GtkAction *action, GtkWidget *callba
                       GCONF_LIST_MODEL_VALUE_COLUMN, &value,
                       -1);
 
-  gconf_client_set (client, path, value, NULL);
+  if (gconf_util_can_edit_mandatory ()) {
+    GConfClient *client;
 
-  gconf_client_suggest_sync (client, &error);
-  if (error) {
-    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    client = gconf_editor_window_get_client (GCONF_EDITOR_WINDOW_TYPE_MANDATORY);
+    if (client != NULL) {
+      GError *error = NULL;
+
+      gconf_client_set (client, path, value, NULL);
+      gconf_client_suggest_sync (client, &error);
+      g_object_unref (client);
+
+      if (!error)
+        return;
+      
+      if (!can_use_pk)
+        gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    }
+  }
+
+  if (can_use_pk) {
+    gconf_client_suggest_sync (gconfwindow->client, NULL);
+    gconf_pk_set_mandatory_async (path,
+                                  GDK_WINDOW_XWINDOW (GTK_WIDGET (window)->window),
+                                  (GFunc) gconf_editor_popup_policykit_callback,
+                                  g_object_ref (window), g_object_unref);
   }
-  g_object_unref (client);
 }
 
 static GtkActionEntry entries[] = {
@@ -905,11 +931,13 @@ list_view_button_press_event (GtkTreeView *tree_view, GdkEventButton *event, GCo
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/EditKey"), 
 						  TRUE);
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/UnsetKey"), 
-						  TRUE);
+						  window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/DefaultKey"), 
-						  TRUE);
+						  (gconf_util_can_edit_defaults () && window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS) ||
+						  (gconf_pk_can_set_default () && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 			gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/MandatoryKey"), 
-						  TRUE);
+						  (gconf_util_can_edit_mandatory () && window->type != GCONF_EDITOR_WINDOW_TYPE_MANDATORY) ||
+						  (gconf_pk_can_set_mandatory () && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 			
 			gtk_tree_path_free (path);
 		}
@@ -1006,11 +1034,13 @@ gconf_editor_window_list_view_popup_menu (GtkWidget *widget, GConfEditorWindow *
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/EditKey"), 
 					  TRUE);
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/UnsetKey"), 
-					  TRUE);
+					  window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS);
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/DefaultKey"), 
-					  TRUE);
+					  (gconf_util_can_edit_defaults () && window->type != GCONF_EDITOR_WINDOW_TYPE_DEFAULTS) ||
+					  (gconf_pk_can_set_default () && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 		gtk_widget_set_sensitive (gtk_ui_manager_get_widget (window->ui_manager, "/GConfKeyPopupMenu/MandatoryKey"), 
-					  TRUE);
+					  (gconf_util_can_edit_mandatory () && window->type != GCONF_EDITOR_WINDOW_TYPE_MANDATORY) ||
+					  (gconf_pk_can_set_mandatory () && window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL));
 			
 	}
 	else {
@@ -1069,7 +1099,7 @@ strip_whitespace (const char *text)
 		p = g_utf8_next_char (p);
 	}
 
-	return str->str;
+	return g_string_free (str, FALSE);
 }
 
 static void
@@ -1271,6 +1301,8 @@ gconf_editor_window_style_set (GtkWidget *widget, GtkStyle *prev_style, GtkWidge
 {
 	gtk_widget_modify_base (text_view, GTK_STATE_NORMAL,
 				&GTK_WIDGET (widget)->style->bg[GTK_STATE_NORMAL]);
+	gtk_widget_modify_text (text_view, GTK_STATE_NORMAL,
+				&GTK_WIDGET (widget)->style->fg[GTK_STATE_NORMAL]);
 }
 
 void
@@ -1373,7 +1405,7 @@ gconf_editor_window_get_property (GObject      *object,
 static void
 gconf_editor_window_finalize (GObject *object)
 {
-        GConfEditorWindow *window = (GConfEditorWindow *) object;
+	GConfEditorWindow *window = GCONF_EDITOR_WINDOW(object);
 	GConfClient       *client;
 
 	client = gconf_client_get_default ();
@@ -1388,15 +1420,21 @@ gconf_editor_window_finalize (GObject *object)
 
 	g_object_unref (client);
 
-	/* FIXME: unref client? */
+	/* for some reason, the program crashes if I try to unref the client for
+         * the other window types.
+         */
+	if (window->type == GCONF_EDITOR_WINDOW_TYPE_NORMAL) {
+	  g_object_unref (window->client);
+        }
 
-        parent_class->finalize (object);
+	G_OBJECT_CLASS (gconf_editor_window_parent_class)->finalize (object);
+    
 }
 
 static void
 gconf_editor_window_class_init (GConfEditorWindowClass *klass)
 {
-	GObjectClass *object_class = (GObjectClass *) klass;
+	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 
 	object_class->set_property = gconf_editor_window_set_property;
 	object_class->get_property = gconf_editor_window_get_property;
@@ -1404,18 +1442,45 @@ gconf_editor_window_class_init (GConfEditorWindowClass *klass)
 
 	g_object_class_install_property (object_class, PROP_TYPE,
 					 g_param_spec_int ("type",
-					 _("Type"),
-					 _("The Configuration Editor window type."),
+					 "Type",
+					 "The Configuration Editor window type.",
 					 GCONF_EDITOR_WINDOW_TYPE_NORMAL, GCONF_EDITOR_WINDOW_TYPE_MANDATORY,
 					 GCONF_EDITOR_WINDOW_TYPE_NORMAL, 
 					 (G_PARAM_WRITABLE |
 					 G_PARAM_CONSTRUCT_ONLY)));
-
-
-	parent_class = g_type_class_peek_parent (klass);
 }
 
+static void
+tree_view_style_set_cb (GtkWidget *widget,
+                        GtkStyle *previous_style,
+                        GObject *cell)
+{
+        GdkPixbuf *folder_open = NULL, *folder_closed = NULL;
+        int w, h;
 
+        if (gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
+                                               GTK_ICON_SIZE_MENU,
+                                               &w, &h)) {
+                GtkIconTheme *theme;
+
+                theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
+                
+                folder_open = gtk_icon_theme_load_icon (theme, "folder-open", w, GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
+                folder_closed = gtk_icon_theme_load_icon (theme, "folder", w, GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
+        }
+
+        g_object_set (cell,
+                      "pixbuf-expander-open", folder_open,
+                      "pixbuf-expander-closed", folder_closed,
+                      "pixbuf", folder_closed,
+                      NULL);
+        if (folder_open) {
+                g_object_unref (folder_open);
+        }
+        if (folder_closed) {
+                g_object_unref (folder_closed);
+        }
+}
 
 static void
 gconf_editor_window_init (GConfEditorWindow *window)
@@ -1514,12 +1579,8 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	column = gtk_tree_view_column_new ();
 
 	cell = gtk_cell_renderer_pixbuf_new ();
+        g_signal_connect (window->tree_view, "style-set", G_CALLBACK (tree_view_style_set_cb), cell);
 	gtk_tree_view_column_pack_start (column, cell, FALSE);
-	gtk_tree_view_column_set_attributes (column, cell,
-					     "pixbuf", GCONF_TREE_MODEL_CLOSED_ICON_COLUMN,
-					     "pixbuf_expander_closed", GCONF_TREE_MODEL_CLOSED_ICON_COLUMN,
-					     "pixbuf_expander_open", GCONF_TREE_MODEL_OPEN_ICON_COLUMN,
-					     NULL);
 	cell = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, cell, TRUE);
 	gtk_tree_view_column_set_attributes (column, cell,
@@ -1561,9 +1622,10 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gtk_tree_view_column_set_title (column, _("Name"));
 
 	cell = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (cell, "stock-size", GTK_ICON_SIZE_MENU, NULL);
 	gtk_tree_view_column_pack_start (column, cell, FALSE);
 	gtk_tree_view_column_set_attributes (column, cell,
-					     "pixbuf", GCONF_LIST_MODEL_ICON_COLUMN,
+					     "icon-name", GCONF_LIST_MODEL_ICON_NAME_COLUMN,
 					     NULL);
 
 	gtk_tree_view_column_set_resizable (column, TRUE);
@@ -1727,27 +1789,3 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gconf_editor_window_setup_ui_prefs_handler (window);
 }
 
-GType
-gconf_editor_window_get_type (void)
-{
-	static GType object_type = 0;
-
-	if (!object_type) {
-		static const GTypeInfo object_info = {
-			sizeof (GConfEditorWindowClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) gconf_editor_window_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (GConfEditorWindow),
-			0,              /* n_preallocs */
-			(GInstanceInitFunc) gconf_editor_window_init
-		};
-
-		object_type = g_type_register_static (GTK_TYPE_WINDOW, "GConfEditorWindow", &object_info, 0);
-	}
-
-	return object_type;
-}
-
