@@ -181,8 +181,12 @@ gconf_list_model_set_root_path (GConfListModel *model, const gchar *root_path)
 
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
 		gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
+
 		gtk_tree_path_free (path);
 	}
+
+	if (values) 
+		g_slist_free (values);
 }
 
 static GtkTreeModelFlags 
@@ -386,8 +390,14 @@ gconf_list_model_finalize (GObject *object)
                 g_object_unref (list_model->client);
         }
 
+	if (list_model->values) {
+		g_slist_foreach (list_model->values, (GFunc) gconf_entry_free, NULL);
+		g_slist_free (list_model->values);
+		list_model->values = NULL;
+	}
+
 	g_free (list_model->root_path);
-	
+
 	G_OBJECT_CLASS (gconf_list_model_parent_class)->finalize (object);
 }
 
