@@ -24,7 +24,7 @@
 #include "gedit-output-window.h"
 #include "gconf-tree-model.h"
 #include "gconf-list-model.h"
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 typedef struct _SearchIter SearchIter;
 
@@ -33,11 +33,13 @@ struct _SearchIter {
 	gboolean search_values;
 	gboolean search_keys;
 	GeditOutputWindow *output_window;
+	GObject *searching;
 	int res;
 };
 
 int gconf_tree_model_build_match_list (GConfTreeModel *tree_model, GeditOutputWindow *output_window,
-				       const char *pattern, gboolean search_keys, gboolean search_values);
+				       const char *pattern, gboolean search_keys, gboolean search_values, 
+				       GObject *dialog);
 
 
 
